package info.aspirecn.rdc.iov.sjjh.servicenode.supplier.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import info.aspirecn.rdc.aspirecloud.node.except.utils.ErrorUtils;
import info.aspirecn.rdc.iov.sjjh.servicenode.supplier.constant.ChannelConstant;
import info.aspirecn.rdc.iov.sjjh.servicenode.supplier.service.TrafficRecordService;
import info.aspirecn.rdc.iov.sjjh.servicenode.supplier.utils.VehicleJsonHelper;
import info.aspirecn.rdc.iov.sjjh.servicenode.supplier.vo.TrafficRecordResponseDetailObject;
import info.aspirecn.rdc.iov.sjjh.servicenode.supplier.vo.TrafficRecordResponseObject;
import lombok.extern.slf4j.Slf4j;
import okhttp3.*;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;

/**
 * @author xusonglin
 * @version V1.0
 **/
@Service
@Slf4j
public class TrafficRecordServiceImpl implements TrafficRecordService {
    @Autowired
    StringRedisTemplate stringRedisTemplate;
    @Autowired
    VehicleJsonHelper vehicleJsonHelper;

    @Value("${supplier.traffic.record.tokenUrl}")
    private String tokenUrl;

    @Value("${supplier.traffic.record.clientKey}")
    private String clientKey;

    @Value("${supplier.traffic.record.clientSecret}")
    private String clientSecret;

    @Value("${supplier.traffic.record.apiUrl}")
    private String apiUrl;

    @Value("${supplier.traffic.record.multipleMonthRecords}")
    private String multipleMonthRecords;

    private OkHttpClient okHttpClient = new OkHttpClient.Builder().build();
    private final int QUERY_TOKEN_OUT_TIME = 1500;
    private final String QUERY_TOKEN_SUCCESS_CODE = "200";
    private static final String MEDIA_TYPE = "application/json;charset=UTF-8";
    private final int TRAFFIC_RECORD_OUT_TIME = 2000;

    @Override
    public String queryToken() {
        Request.Builder builder = new Request.Builder();
        Request okRequest = new Request.Builder()
                .get()
                .url(tokenUrl)
                .build();
        HttpUrl.Builder urlBuilder = okRequest.url().newBuilder();
        urlBuilder.addQueryParameter("clientKey", clientKey);
        urlBuilder.addQueryParameter("clientSecret", clientSecret);
        builder.url(urlBuilder.build());

        OkHttpClient client = okHttpClient.newBuilder()
                .connectTimeout(QUERY_TOKEN_OUT_TIME, TimeUnit.MILLISECONDS)
                .readTimeout(QUERY_TOKEN_OUT_TIME, TimeUnit.MILLISECONDS)
                .writeTimeout(QUERY_TOKEN_OUT_TIME, TimeUnit.MILLISECONDS)
                .build();
        String responseContext;
        String token = "";
        try {
            Response response = client.newCall(builder.build()).execute();
            responseContext = response.body().string();
            log.info("supplier10000036.queryToken接口responseContext={}", responseContext);
        } catch (IOException ioe) {
            ErrorUtils.captureException(ioe);
            log.info("supplier10000036.queryToken接口SocketTimeoutException=", ioe);
            return token;
        }
        JSONObject responseObject = JSON.parseObject(responseContext);
        if (responseObject != null) {
            String code = responseObject.getString("code");
            if (code.equals(QUERY_TOKEN_SUCCESS_CODE)) {
                token = responseObject.getString("message");
                stringRedisTemplate.opsForValue().set(ChannelConstant.TRAFFIC_QUERY_TOKEN, token, 1, TimeUnit.HOURS);
                return token;
            } else {
                return token;
            }
        } else {
            return token;
        }
    }

    private String getToken() {
        String token = stringRedisTemplate.opsForValue().get(ChannelConstant.TRAFFIC_QUERY_TOKEN);
        if (StringUtils.isBlank(token)) {
            token = queryToken();
        }
        return token;
    }

    @Override
    public Integer trafficRecordQuery(String plateNum, String plateColor) {
        int score = 0;
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyyMM");
        Calendar calendar = Calendar.getInstance();
        calendar.add(Calendar.MONTH, -1);
        String endTime = simpleDateFormat.format(calendar.getTime());
        calendar.add(Calendar.MONTH, -2);
        String startTime = simpleDateFormat.format(calendar.getTime());

        JSONObject requestObject = new JSONObject();
        requestObject.put("plateNumber", plateNum + "_" + plateColor);
        requestObject.put("startMonth", startTime);
        requestObject.put("endMonth", endTime);


        MediaType mediaType = MediaType.parse(MEDIA_TYPE);
        RequestBody requestBody = RequestBody.create(mediaType, requestObject.toJSONString());
        Request okRequest = new Request.Builder()
                .post(requestBody)
                .header("X-TOKEN", getToken())
                .url(apiUrl + multipleMonthRecords)
                .build();
        OkHttpClient client = okHttpClient.newBuilder()
                .connectTimeout(TRAFFIC_RECORD_OUT_TIME, TimeUnit.MILLISECONDS)
                .readTimeout(TRAFFIC_RECORD_OUT_TIME, TimeUnit.MILLISECONDS)
                .writeTimeout(TRAFFIC_RECORD_OUT_TIME, TimeUnit.MILLISECONDS)
                .build();
        String responseContext = "";
        try {
            Response response = client.newCall(okRequest).execute();
            if (response.body() != null) {
                responseContext = response.body().string();
            }
            log.info("supplier10000036.trafficRecordQuery接口responseContext={}", responseContext);
        } catch (IOException ioe) {
            ErrorUtils.captureException(ioe);
            log.info("supplier10000036.trafficRecordQuery接口SocketTimeoutException=", ioe);
            return score;
        }
        if (StringUtils.isNotBlank(responseContext)) {
            TrafficRecordResponseObject responseObject = JSON.toJavaObject(JSON.parseObject(responseContext),
                    TrafficRecordResponseObject.class);
            if (responseObject != null) {
                return getTrafficRecordScore(responseObject);
            } else {
                return score;
            }
        } else {
            return score;
        }
    }

    private Integer getTrafficRecordScore(TrafficRecordResponseObject responseObject) {
        if (!responseObject.getCode().equals(ChannelConstant.TRAFFIC_QUERY_SUCCESS_CODE)) {
            return 0;
        }

        List<TrafficRecordResponseDetailObject> detailObjectList = JSON.parseArray(responseObject.getMessage(),
                TrafficRecordResponseDetailObject.class);

        double score = 0;
        for (TrafficRecordResponseDetailObject detailObject : detailObjectList) {
            Map<String, String> millageMap = vehicleJsonHelper.getMillage();
            String scoreString = millageMap.get(detailObject.getSumFeeMileage());
            score = score + Integer.parseInt(scoreString);
        }
        return (int) Math.round(score / detailObjectList.size());
    }
}
